@@ -26,8 +26,8 @@
  */
 package org.apache.http.osgi.impl;
 
+import java.util.List;
 import java.util.Locale;
-import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -37,8 +37,6 @@ import org.apache.http.HttpRequest;
 import org.apache.http.impl.conn.DefaultRoutePlanner;
 import org.apache.http.osgi.services.ProxyConfiguration;
 import org.apache.http.protocol.HttpContext;
-import org.osgi.framework.BundleContext;
-import org.osgi.framework.ServiceRegistration;
 
 /**
  * @since 4.3
@@ -55,16 +53,11 @@ final class OSGiHttpRoutePlanner extends DefaultRoutePlanner {
                                                                   "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
                                                                   "([01]?\\d\\d?|2[0-4]\\d|25[0-5])$");
 
-    private final BundleContext bundleContext;
+    private final List<ProxyConfiguration> proxyConfigurations;
 
-    private final Map<String, ServiceRegistration> registeredConfigurations;
-
-    public OSGiHttpRoutePlanner(
-            final BundleContext bundleContext,
-            final Map<String, ServiceRegistration> registeredConfigurations) {
+    public OSGiHttpRoutePlanner(final List<ProxyConfiguration> proxyConfigurations) {
         super(null);
-        this.bundleContext = bundleContext;
-        this.registeredConfigurations = registeredConfigurations;
+        this.proxyConfigurations = proxyConfigurations;
     }
 
     /**
@@ -72,22 +65,17 @@ final class OSGiHttpRoutePlanner extends DefaultRoutePlanner {
      */
     @Override
     protected HttpHost determineProxy(final HttpHost target, final HttpRequest request, final HttpContext context) throws HttpException {
-        ProxyConfiguration proxyConfiguration = null;
         HttpHost proxyHost = null;
-        for (final ServiceRegistration registration : registeredConfigurations.values()) {
-            final Object proxyConfigurationObject = bundleContext.getService(registration.getReference());
-            if (proxyConfigurationObject != null) {
-                proxyConfiguration = (ProxyConfiguration) proxyConfigurationObject;
-                if (proxyConfiguration.isEnabled()) {
-                    for (final String exception : proxyConfiguration.getProxyExceptions()) {
-                        if (createMatcher(exception).matches(target.getHostName())) {
-                            return null;
-                        }
-                    }
-                    if (null == proxyHost) {
-                        proxyHost = new HttpHost(proxyConfiguration.getHostname(), proxyConfiguration.getPort());
+        for (final ProxyConfiguration proxyConfiguration : proxyConfigurations) {
+            if (proxyConfiguration.isEnabled()) {
+                for (final String exception : proxyConfiguration.getProxyExceptions()) {
+                    if (createMatcher(exception).matches(target.getHostName())) {
+                        return null;
                     }
                 }
+                if (null == proxyHost) {
+                    proxyHost = new HttpHost(proxyConfiguration.getHostname(), proxyConfiguration.getPort());
+                }
             }
         }
 
