@@ -28,12 +28,14 @@ package org.apache.http.client.utils;
 
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
+import java.util.EnumSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Stack;
 
 import org.apache.http.HttpHost;
-import org.apache.http.annotation.Immutable;
 import org.apache.http.conn.routing.RouteInfo;
 import org.apache.http.util.Args;
 import org.apache.http.util.TextUtils;
@@ -44,9 +46,46 @@ import org.apache.http.util.TextUtils;
  *
  * @since 4.0
  */
-@Immutable
 public class URIUtils {
 
+    /**
+     * Flags that control how URI is being rewritten.
+     *
+     * @since 4.5.8
+     */
+    public enum UriFlag {
+        DROP_FRAGMENT,
+        NORMALIZE
+    }
+
+    /**
+     * Empty set of uri flags.
+     *
+     * @since 4.5.8
+     */
+    public static final EnumSet<UriFlag> NO_FLAGS = EnumSet.noneOf(UriFlag.class);
+
+    /**
+     * Set of uri flags containing {@link UriFlag#DROP_FRAGMENT}.
+     *
+     * @since 4.5.8
+     */
+    public static final EnumSet<UriFlag> DROP_FRAGMENT = EnumSet.of(UriFlag.DROP_FRAGMENT);
+
+    /**
+     * Set of uri flags containing {@link UriFlag#NORMALIZE}.
+     *
+     * @since 4.5.8
+     */
+    public static final EnumSet<UriFlag> NORMALIZE = EnumSet.of(UriFlag.NORMALIZE);
+
+    /**
+     * Set of uri flags containing {@link UriFlag#DROP_FRAGMENT} and {@link UriFlag#NORMALIZE}.
+     *
+     * @since 4.5.8
+     */
+    public static final EnumSet<UriFlag> DROP_FRAGMENT_AND_NORMALIZE = EnumSet.of(UriFlag.DROP_FRAGMENT, UriFlag.NORMALIZE);
+
      /**
          * Constructs a {@link URI} using all the parameters. This should be
          * used instead of
@@ -127,12 +166,40 @@ public class URIUtils {
      *
      * @throws URISyntaxException
      *             If the resulting URI is invalid.
+     * @deprecated (4.5.8) Use {@link #rewriteURI(URI, HttpHost, EnumSet)}
+     */
+    @Deprecated
+    public static URI rewriteURI(
+            final URI uri,
+            final HttpHost target,
+            final boolean dropFragment) throws URISyntaxException
+    {
+        return rewriteURI(uri, target, dropFragment ? DROP_FRAGMENT : NO_FLAGS);
+    }
+
+    /**
+     * A convenience method for creating a new {@link URI} whose scheme, host
+     * and port are taken from the target host, but whose path, query and
+     * fragment are taken from the existing URI. What exactly is used and how
+     * is driven by the passed in flags. The path is set to "/" if not explicitly specified.
+     *
+     * @param uri
+     *            Contains the path, query and fragment to use.
+     * @param target
+     *            Contains the scheme, host and port to use.
+     * @param flags
+     *            True if the fragment should not be copied.
+     *
+     * @throws URISyntaxException
+     *             If the resulting URI is invalid.
+     * @since 4.5.8
      */
     public static URI rewriteURI(
             final URI uri,
             final HttpHost target,
-            final boolean dropFragment) throws URISyntaxException {
+            final EnumSet<UriFlag> flags) throws URISyntaxException {
         Args.notNull(uri, "URI");
+        Args.notNull(flags, "URI flags");
         if (uri.isOpaque()) {
             return uri;
         }
@@ -146,24 +213,37 @@ public class URIUtils {
             uribuilder.setHost(null);
             uribuilder.setPort(-1);
         }
-        if (dropFragment) {
+        if (flags.contains(UriFlag.DROP_FRAGMENT)) {
             uribuilder.setFragment(null);
         }
-        if (TextUtils.isEmpty(uribuilder.getPath())) {
-            uribuilder.setPath("/");
+        if (flags.contains(UriFlag.NORMALIZE)) {
+            final List<String> originalPathSegments = uribuilder.getPathSegments();
+            final List<String> pathSegments = new ArrayList<String>(originalPathSegments);
+            for (final Iterator<String> it = pathSegments.iterator(); it.hasNext(); ) {
+                final String pathSegment = it.next();
+                if (pathSegment.isEmpty() && it.hasNext()) {
+                    it.remove();
+                }
+            }
+            if (pathSegments.size() != originalPathSegments.size()) {
+                uribuilder.setPathSegments(pathSegments);
+            }
+        }
+        if (uribuilder.isPathEmpty()) {
+            uribuilder.setPathSegments("");
         }
         return uribuilder.build();
     }
 
     /**
      * A convenience method for
-     * {@link URIUtils#rewriteURI(URI, HttpHost, boolean)} that always keeps the
+     * {@link URIUtils#rewriteURI(URI, HttpHost, EnumSet)} that always keeps the
      * fragment.
      */
     public static URI rewriteURI(
             final URI uri,
             final HttpHost target) throws URISyntaxException {
-        return rewriteURI(uri, target, false);
+        return rewriteURI(uri, target, NORMALIZE);
     }
 
     /**
@@ -186,6 +266,9 @@ public class URIUtils {
         if (uribuilder.getUserInfo() != null) {
             uribuilder.setUserInfo(null);
         }
+        if (uribuilder.getPathSegments().isEmpty()) {
+            uribuilder.setPathSegments("");
+        }
         if (TextUtils.isEmpty(uribuilder.getPath())) {
             uribuilder.setPath("/");
         }
@@ -208,25 +291,32 @@ public class URIUtils {
      * @since 4.4
      */
     public static URI rewriteURIForRoute(final URI uri, final RouteInfo route) throws URISyntaxException {
+        return rewriteURIForRoute(uri, route, true);
+    }
+
+    /**
+     * A convenience method that optionally converts the original {@link java.net.URI} either
+     * to a relative or an absolute form as required by the specified route.
+     *
+     * @param uri
+     *            original URI.
+     * @throws URISyntaxException
+     *             If the resulting URI is invalid.
+     *
+     * @since 4.5.8
+     */
+    public static URI rewriteURIForRoute(final URI uri, final RouteInfo route, final boolean normalizeUri) throws URISyntaxException {
         if (uri == null) {
             return null;
         }
         if (route.getProxyHost() != null && !route.isTunnelled()) {
             // Make sure the request URI is absolute
-            if (!uri.isAbsolute()) {
-                final HttpHost target = route.getTargetHost();
-                return rewriteURI(uri, target, true);
-            } else {
-                return rewriteURI(uri);
-            }
-        } else {
-            // Make sure the request URI is relative
-            if (uri.isAbsolute()) {
-                return rewriteURI(uri, null, true);
-            } else {
-                return rewriteURI(uri);
-            }
+            return uri.isAbsolute()
+                    ? rewriteURI(uri)
+                    : rewriteURI(uri, route.getTargetHost(), normalizeUri ? DROP_FRAGMENT_AND_NORMALIZE : DROP_FRAGMENT);
         }
+        // Make sure the request URI is relative
+        return uri.isAbsolute() ? rewriteURI(uri, null, normalizeUri ? DROP_FRAGMENT_AND_NORMALIZE : DROP_FRAGMENT) : rewriteURI(uri);
     }
 
     /**
@@ -270,7 +360,7 @@ public class URIUtils {
         }
         try {
             return normalizeSyntax(resolved);
-        } catch (URISyntaxException ex) {
+        } catch (final URISyntaxException ex) {
             throw new IllegalArgumentException(ex);
         }
     }
@@ -281,39 +371,32 @@ public class URIUtils {
      *
      * @param uri the original URI
      * @return the URI without dot segments
+     *
+     * @since 4.5
      */
-    static URI normalizeSyntax(final URI uri) throws URISyntaxException {
+    public static URI normalizeSyntax(final URI uri) throws URISyntaxException {
         if (uri.isOpaque() || uri.getAuthority() == null) {
             // opaque and file: URIs
             return uri;
         }
-        Args.check(uri.isAbsolute(), "Base URI must be absolute");
         final URIBuilder builder = new URIBuilder(uri);
-        final String path = builder.getPath();
-        if (path != null && !path.equals("/")) {
-            final String[] inputSegments = path.split("/");
-            final Stack<String> outputSegments = new Stack<String>();
-            for (final String inputSegment : inputSegments) {
-                if ((inputSegment.isEmpty()) || (".".equals(inputSegment))) {
-                    // Do nothing
-                } else if ("..".equals(inputSegment)) {
-                    if (!outputSegments.isEmpty()) {
-                        outputSegments.pop();
-                    }
-                } else {
-                    outputSegments.push(inputSegment);
+        final List<String> inputSegments = builder.getPathSegments();
+        final Stack<String> outputSegments = new Stack<String>();
+        for (final String inputSegment : inputSegments) {
+            if (".".equals(inputSegment)) {
+                // Do nothing
+            } else if ("..".equals(inputSegment)) {
+                if (!outputSegments.isEmpty()) {
+                    outputSegments.pop();
                 }
+            } else {
+                outputSegments.push(inputSegment);
             }
-            final StringBuilder outputBuffer = new StringBuilder();
-            for (final String outputSegment : outputSegments) {
-                outputBuffer.append('/').append(outputSegment);
-            }
-            if (path.lastIndexOf('/') == path.length() - 1) {
-                // path.endsWith("/") || path.equals("")
-                outputBuffer.append('/');
-            }
-            builder.setPath(outputBuffer.toString());
         }
+        if (outputSegments.size() == 0) {
+            outputSegments.add("");
+        }
+        builder.setPathSegments(outputSegments);
         if (builder.getScheme() != null) {
             builder.setScheme(builder.getScheme().toLowerCase(Locale.ROOT));
         }
@@ -381,7 +464,7 @@ public class URIUtils {
             if (!TextUtils.isBlank(host)) {
                 try {
                     target = new HttpHost(host, port, scheme);
-                } catch (IllegalArgumentException ignore) {
+                } catch (final IllegalArgumentException ignore) {
                 }
             }
         }
