@@ -29,6 +29,8 @@ package org.apache.http.client.utils;
 
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 
 import org.apache.http.Consts;
@@ -37,6 +39,7 @@ import org.apache.http.entity.ContentType;
 import org.apache.http.entity.StringEntity;
 import org.apache.http.message.BasicNameValuePair;
 import org.apache.http.protocol.HTTP;
+import org.hamcrest.CoreMatchers;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -102,6 +105,42 @@ public class TestURLEncodedUtils {
         assertNameValuePair(result.get(1), "d", "e");
     }
 
+    @Test
+    public void testParseSegments() throws Exception {
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("/this/that"),
+                CoreMatchers.equalTo(Arrays.asList("this", "that")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("this/that"),
+                CoreMatchers.equalTo(Arrays.asList("this", "that")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("this//that"),
+                CoreMatchers.equalTo(Arrays.asList("this", "", "that")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("this//that/"),
+                CoreMatchers.equalTo(Arrays.asList("this", "", "that", "")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("this//that/%2fthis%20and%20that"),
+                CoreMatchers.equalTo(Arrays.asList("this", "", "that", "/this and that")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("this///that//"),
+                CoreMatchers.equalTo(Arrays.asList("this", "", "", "that", "", "")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments("/"),
+                CoreMatchers.equalTo(Collections.singletonList("")));
+        Assert.assertThat(URLEncodedUtils.parsePathSegments(""),
+                CoreMatchers.equalTo(Collections.<String>emptyList()));
+    }
+
+    @Test
+    public void testFormatSegments() throws Exception {
+        Assert.assertThat(URLEncodedUtils.formatSegments("this", "that"),
+                CoreMatchers.equalTo("/this/that"));
+        Assert.assertThat(URLEncodedUtils.formatSegments("this", "", "that"),
+                CoreMatchers.equalTo("/this//that"));
+        Assert.assertThat(URLEncodedUtils.formatSegments("this", "", "that", "/this and that"),
+                CoreMatchers.equalTo("/this//that/%2Fthis%20and%20that"));
+        Assert.assertThat(URLEncodedUtils.formatSegments("this", "", "", "that", "", ""),
+                CoreMatchers.equalTo("/this///that//"));
+        Assert.assertThat(URLEncodedUtils.formatSegments(""),
+                CoreMatchers.equalTo("/"));
+        Assert.assertThat(URLEncodedUtils.formatSegments(),
+                CoreMatchers.equalTo(""));
+    }
+
     @Test
     public void testParseURLCodedContentString() throws Exception {
         List <NameValuePair> result;
